@@ -1,7 +1,7 @@
 package Perinci::Sub::Wrapper;
 
-our $DATE = '2016-01-22'; # DATE
-our $VERSION = '0.81'; # VERSION
+our $DATE = '2016-02-04'; # DATE
+our $VERSION = '0.82'; # VERSION
 
 use 5.010001;
 use strict;
@@ -9,6 +9,7 @@ use warnings;
 use experimental 'smartmatch';
 use Log::Any::IfLOG '$log';
 
+use Data::Dmp qw(dmp);
 use Function::Fallback::CoreOrPP qw(clone);
 use Perinci::Sub::Normalize qw(normalize_function_metadata);
 use Perinci::Sub::Util qw(err);
@@ -36,14 +37,6 @@ sub new {
     bless {}, $class;
 }
 
-sub __squote {
-    require Data::Dumper;
-    my $res = Data::Dumper->new([shift])->
-        Purity(1)->Terse(1)->Deepcopy(1)->Indent(0)->Dump;
-    chomp $res;
-    $res;
-}
-
 sub _check_module {
     my ($self, $mod) = @_;
 
@@ -119,7 +112,7 @@ sub _add_var {
     unless (exists $self->{_vars}{$var}) {
         local $self->{_cur_section};
         $self->select_section('declare_vars');
-        $self->push_lines("my \$$var = ".__squote($value).";");
+        $self->push_lines("my \$$var = ".dmp($value).";");
         $self->{_vars}{$var} = $value;
     }
 }
@@ -506,7 +499,7 @@ sub handle_args_as {
         $self->select_section('ACCEPT_ARGS2');
         for my $a (sort keys %$args_p) {
             my $as = $args_p->{$a};
-            my $line = '$args{'.__squote($a).'} = ';
+            my $line = '$args{'.dmp($a).'} = ';
             defined($as->{pos}) or die "Error in args property for arg '$a': ".
                 "no pos defined";
             my $pos = int($as->{pos} + 0);
@@ -554,7 +547,7 @@ sub handle_args_as {
         for my $a (sort {$args_p->{$a}{pos} <=> $args_p->{$b}{pos}}
                        keys %$args_p) {
             my $as = $args_p->{$a};
-            my $t = '$args{'.__squote($a).'}';
+            my $t = '$args{'.dmp($a).'}';
             my $line;
             defined($as->{pos}) or die "Error in args property for arg '$a': ".
                 "no pos defined";
@@ -612,7 +605,7 @@ sub _handle_args {
         $self->_errif(400, q["Invalid argument name (please use letters/numbers/underscores only)'].$prefix.q[$_'"],
                       '!/\A(-?)\w+(\.\w+)*\z/o');
         $self->_errif(400, q["Unknown argument '].$prefix.q[$_'"],
-                      '!($1 || $_ ~~ '.__squote([sort keys %$v]).')');
+                      '!($1 || $_ ~~ '.dmp([sort keys %$v]).')');
         $self->unindent;
         $self->push_lines('}');
     }
@@ -641,6 +634,11 @@ sub _handle_args {
                 $self->indent;
 
                 if ($argspec->{stream}) {
+                    die "Error in schema for argument '$argname': must be str/buf/array if stream=1"
+                        unless $sch->[0] =~ /\A(str|buf|array)\z/; # XXX allow 'any' if all of its 'of' values are str/buf/array
+                    die "Error in schema for argument '$argname': must specify 'of' array clause if stream=1"
+                        if $sch->[0] eq 'array' && !$sch->[1]{of};
+
                     $self->_errif(
                         400,
                         qq["Argument '$prefix$argname' (stream) fails validation: must be coderef"],
@@ -666,7 +664,7 @@ sub _handle_args {
                 my $cd = $self->_plc->compile(
                     data_name            => $dn,
                     data_term            => $argspec->{stream} ? '$rec' : $argterm,
-                    schema               => $sch,
+                    schema               => $argspec->{stream} && $sch->[0] eq 'array' ? $sch->[1]{of} : $sch,
                     schema_is_normalized => $opt_sin,
                     return_type          => 'str',
                     indent_level         => $self->get_indent_level + 1,
@@ -732,11 +730,11 @@ sub _handle_args {
                 if ($has_default_prop) {
                     $self->push_lines(
                         '} else {',
-                        "    $argterm //= ".__squote($argspec->{default}).";");
+                        "    $argterm //= ".dmp($argspec->{default}).";");
                 } elsif ($has_sch_default) {
                     $self->push_lines(
                         '} else {',
-                        "    $argterm //= ".__squote($sch->[1]{default}).";");
+                        "    $argterm //= ".dmp($sch->[1]{default}).";");
                 }
                 $self->push_lines("} ## if exists arg $prefix$argname");
             } # if opt_va
@@ -744,7 +742,7 @@ sub _handle_args {
         } elsif ($has_default_prop) {
             # doesn't have schema but have 'default' property, we still need to
             # set default here
-            $self->push_lines("$argterm = ".__squote($argspec->{default}).
+            $self->push_lines("$argterm = ".dmp($argspec->{default}).
                                   " if !exists($argterm);");
         }
         if ($argspec->{req} && $opt_va) {
@@ -895,11 +893,18 @@ sub handle_result {
         );
         for my $s (sort keys %schemas_by_status) {
             my $sch = $schemas_by_status{$s};
+            if ($v->{stream}) {
+                die "Error in result schema: must be str/buf/array if stream=1"
+                    unless $sch->[0] =~ /\A(str|buf|array)\z/; # XXX allow 'any' if all of its 'of' values are str/buf/array
+                die "Error in result schema: must specify 'of' array clause if stream=1"
+                    if $sch->[0] eq 'array' && !$sch->[1]{of};
+            }
             $self->push_lines("if (\$_w_res->[0] == $s) {");
             $self->indent;
             $self->push_lines('if (!$_w_res_is_stream) {');
             $self->indent;
 
+            # validation for when not a stream
             my $cd = $self->_plc->compile(
                 data_name            => '_w_res2',
                 # err_res can clash on arg named 'res'
@@ -944,7 +949,7 @@ sub handle_result {
                 data_name            => 'rec',
                 # err_res can clash on arg named 'res'
                 err_term             => '$rec_err',
-                schema               => $sch,
+                schema               => $sch->[0] eq 'array' ? $sch->[1]{of} : $sch,
                 schema_is_normalized => $opt_sin,
                 return_type          => 'str',
                 indent_level         => $self->get_indent_level + 1,
@@ -1476,7 +1481,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-This document describes version 0.81 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2016-01-22.
+This document describes version 0.82 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2016-02-04.
 
 =head1 SYNOPSIS
 
